#!/usr/bin/perl
#
# Updates the /etc/localtime file as root.
# 
# Usage: set_localtime.pl <$prefix/etc/localtime> </usr/share/zoneinfo/*>

my ($localtime,$zone) = (@ARGV);
die "Invalid parameters." unless (defined $localtime && defined $zone);

# Debug logging
open (LOG, ">>/tmp/set_localtime.log");
print LOG localtime() . ": localtime = $localtime; zone: $zone\n";

# Do some sanity checks on the args.
if ($localtime !~ /\/etc\/localtime$/i) {
	die "$localtime doesn't look like a system localtime path!";
}
if ($zone !~ /^\/usr\/share\/zoneinfo/i) {
	die "$zone doesn't look like a time zone file!";
}

# Back up /etc/localtime in case anything fails.
print LOG localtime() . ": backing up $localtime -> $localtime.unlink\n";
rename($localtime, "$localtime.unlink") or die "Can't rename $localtime: $@ $!";

# Delete /etc/localtime.
unlink($localtime);

# Symlink the time zone over.
if (symlink($zone, $localtime)) {
	# Delete the /etc/localtime backup.
	print LOG localtime() . ": Success!\n";
	unlink("$localtime.unlink");
	exit(0);
}
else {
	# Error! Revert!
	print LOG localtime() . ": error making a symlink: $@ $!\n";
	rename("$localtime.unlink", $localtime) or die "Can't rename $localtime.link: $@ $!";
	die "Couldn't create symlink $zone -> $localtime: $@ $!";
}
